/** @file  lm92.c
 *  @brief LM92 I2C temperature sensor 
 */
#include "lm92.h"
#include "i2c.h"
#include "rs232.h"

/** @brief Get LM92 temperature register
 *  @return : D15..D3 = temperature (1 bit = 0.0625C)
 */
u16 
lm92_read_temperature()
{
	u16 temperature = 0;

	#ifdef DEBUG_RS232
		tx232_puts("lm92read\nr");
	#endif

	
	// Set pointer to 0
	
	
	// Read value
	i2c_start();
	i2c_write(LM92_ADR | I2C_READ);
	
	// MSB ack
	temperature  = i2c_read(1) << 8;	

	// LSB no ack
	temperature |= i2c_read(0);		
	
	i2c_stop();
	
	return temperature;
}


s8 
lm92_get_signed_temperature()
{
	s8 tempS;
	u16 temp = 	lm92_read_temperature() & ~(0xFFF8);
	// D3 : 1 = 0.0625C
	// D7 : 1 = 1C

	tempS = temp>>8;

	// negative temperatur
	if (temp&0x8000)
	{
		tempS = -tempS;
	}
	return tempS;
}

