/** @file  rs232.cpp
 *  @brief RS232 uart
 */
#include "rs232.h"

//========================================================================================
//  Port Serie RS232   8bits, no parity, 1 bit de stop 
//========================================================================================
void
init232(char debit)
{
	
	// Baud Rate = Fosc/(16(SPBRG+1))
	SPBRG = debit;   					
	// 103 : 9600bds, 51 : 19200bds, 16 : 57600bds, 8: 115kbds, 255 : 1Mbps!!! pour 16MHz
	// 155 : 9600bds, 77 : 19200bds, 25 : 57600bds, 12 : 115kbds pour 24MHz
	TXSTA = 0x24; //0b00100100;     		// Asynchronous, 8bits, TXEN, Asynchronous, High speed
	RCSTA = 0x90; //0b10010000;			// SPEN : enable port, CREN : reception continue
	rx232();										// correction bug bootloader (sinon port bloque ?)
}


void
tx232(char val)
{
	while(TXSTAbits.TRMT==0); 					// attendre que le tampon soit vide
	TXREG = val;						// emet le caractere
}


char
rx232(void)
{
	unsigned char tmp; // #### UTILE ??? cf doc MidRange
	
	//overrun Error
	if (RCSTAbits.OERR==1) 
	{ 					
		RCSTAbits.CREN = 0;
		RCSTAbits.CREN = 1;
		tmp = RCREG;
	}
	PIR1bits.RCIF = 0;							// reset flag ISR
	return RCREG;						// retourne le caractere recu
}


// ecrit une string de caracteres sur le port serie
void
tx232_puts(const char * s)
{
	while(*s)
	{
		tx232(*s++);
	}
}
 
 
// detection de reception
char
rx232_detc(void)
{
	while(!PIR1bits.RCIF);
	return rx232();
}


// ecrit 2 caracteres ascii
void
tx232_2ascii(unsigned char car)
{
	tx232(hexa2ascii(car >> 4));
	tx232(hexa2ascii(car & 0x0F));
}

//========================================================================================
// convertit un nombre hexadecimal en ascii
//========================================================================================
unsigned char
hexa2ascii(unsigned char tmp)
{
	tmp += 0x30;
	if (tmp>0x39) tmp+=7;
	return tmp;
}

//========================================================================================
// convertit un nombre ascii en hexa
//========================================================================================
unsigned char
ascii2hexa(unsigned char tmp)
{
	tmp -= 0x30;
	if (tmp>0x09) tmp-=7;
	return tmp;
}
